#include "dronecell.h"
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//!!!!!!!!!!! BE SURE TO DEFINE YOUR HARDWARE AND NETWORK SETTINGS IN dronecell.h !!!!!!!!!!!!
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

/* --------------- VERSION INFORMATION ----------
 V1.00  - 6/2/2010  - Two way TCP/UDP communication
 						Phone Dial , no receive
						Text Send , no receive
-------------------------------------------------*/	

// you must call this function once and supply the UART sending and receiving functions
// DroneCellInit(uart0SendByte,uart0GetByte,uart0FlushReceiveBuffer);
// Be sure to configure uartInit before setting the pointers
void DroneCell_Init( void (*putchar_func)(unsigned char c) , int (*getchar_func)(void) , void (*flush_buffer)(void) ) { // need flush buffer stuff
	SendByte = putchar_func;
	RawGetByte = getchar_func;
	FlushBuffer = flush_buffer;
}

int GetByte(unsigned char seconds) {
	unsigned int long counter=0;
	// FREQ_CPU/180 = 1 second . !!!! DO NOT MODIFY ANYTHING HERE!!!!
	int char_byte = RawGetByte();  // needs to be unsigned

	while( char_byte == -1 && counter < (FREQ_CPU/180)*seconds ) { 
		char_byte = RawGetByte();
		counter++;
	}
	if (counter >= (FREQ_CPU/180)*seconds ) 
		return TIMEOUT_ERROR;

	return char_byte;
}

int CGDCONT(void) {
	int response[1];
	rprintfInit(SendByte);
	rprintf("AT+CGDCONT=1,\"");
	rprintfStr(CGDCONT_PDP);rprintf("\",\"");
	rprintfStr(CGDCONT_APN);rprintf("\"\r");
	FlushBuffer();
	for (int i=0;i<2;++i) { 
		response[i]=GetByte(1); 
		if (response[i] == TIMEOUT_ERROR)
			return TIMEOUT_ERROR;
	} // Get the two byte response
	return response[0]; 
} //end sub


int CSTT(void) {
	int response[1];

	rprintfInit(SendByte); 
	FlushBuffer();
	rprintf("AT+CSTT=\"");
	rprintfStr(CSTT_APN);	rprintf("\",\"");
	rprintfStr(CSTT_USER);	rprintf("\",\"");
	rprintfStr(CSTT_PASS); 	rprintf("\"\r");
	FlushBuffer();
	for (int i=0;i<2;++i) {  
		response[i]=GetByte(2);  
		if (response[i] == TIMEOUT_ERROR)
			return TIMEOUT_ERROR;
	} // Get the two byte response
	return response[0]; 
} //end sub

int CIPHEAD(void) { // Enable +IPD header on receive packets
	int response[1];
	rprintfInit(SendByte); rprintf("AT+CIPHEAD=1\r"); FlushBuffer();

	for (int i=0;i<2;++i) { 
		response[i]=GetByte(2);
		if (response[i] == TIMEOUT_ERROR)
			return TIMEOUT_ERROR;
	} //end for
	return response[0];
}

int CIICR(void) {
	int response[1];
	rprintfInit(SendByte); rprintf("AT+CIICR\r"); FlushBuffer();
	
	response[0]= GetByte(60);  // RawGetByte()  returns -1 if no byte
	if (response[0] == TIMEOUT_ERROR) 
		return TIMEOUT_ERROR;

	response[1]=GetByte(1);
	if (response[1] == TIMEOUT_ERROR)
		return TIMEOUT_ERROR;

	return response[0]; 
} // end sub



int CIFSR(void) {
	int response;
	rprintfInit(SendByte); rprintf("AT+CIFSR\r"); FlushBuffer();
	
	response=0;
	for (int i=0;i<3;++i) {
		while ( response != '.' ) {
			response = GetByte(2);
			if (response == TIMEOUT_ERROR)
				return TIMEOUT_ERROR;
		 }  // get the three dots of an IP address x.x.x.x
	}

	while ( response != 10 ) { 
		response = GetByte(2);
		if (response == TIMEOUT_ERROR)
			return TIMEOUT_ERROR;
	} // get the last 10 = \n

	return OK;
} //end sub



int ConnectServer(char protocol[],char ip_address[],char port[]) {  		// Connect to Server
	int response[7];

	rprintfInit(SendByte); 

	rprintf("AT+CIPSTART=\""); 
	rprintfStr(protocol);	rprintf("\",\"");
	rprintfStr(ip_address); rprintf("\",\"");
	rprintfStr(port);		rprintf("\"\r");
	FlushBuffer();

	for (int i=0;i<2;++i) {  //Get initial , OK or ERROR response
		response[i]=GetByte(1);
		if (response[i] == TIMEOUT_ERROR)
			return TIMEOUT_ERROR;
	} //end for

	if ( response[0] != 48 )
		return ERROR;
	
	response[3]=GetByte(30);  // the secondary response, this is whether it actually connected
	if (response[3] == TIMEOUT_ERROR)
		return TIMEOUT_ERROR;

	for (int i=4;i<6;++i) { 
		response[i]=GetByte(1); 
		if (response[i] == TIMEOUT_ERROR) 
			return TIMEOUT_ERROR;
	} //end for

	if (response[5] == 56 ) {			// 8 is OK, otherwise its bad. Might be other errors tho..
		for (int i=0;i<2;++i) {  // 13- 10
			response[i]=GetByte(1);
			if (response[i] == TIMEOUT_ERROR)
				return TIMEOUT_ERROR;
		} //end for
			return OK; 
	}// end if
	else 
		return ERROR; 
} // end sub 

int Connect_Status(void) {  // returns 1 if good, anything else is bad
	int response[15];
	rprintfInit(SendByte); rprintf("AT+CREG?\r"); FlushBuffer();

	for (int i=0;i<16;++i) {  // Get the headings and everything
		response[i]=GetByte(5);
		if (response[i] == TIMEOUT_ERROR) 
				return ERROR;
	} //end for
	return response[11]; 	// "1" or decimal 49 means good connect
}

void Toggle_PWRKEY(void) { // Some sort of setting to bring it high on startup?
	port_off( PWRKEY_PORT , PWRKEY_PIN); // Source PWRKEY LOW
	delay_ms(2500); // delay 2 seconds
	port_on( PWRKEY_PORT , PWRKEY_PIN); // Pull PWRKEY HIGH
	delay_ms(750);
}

int Read_STATUS(void) { // 0 when not connected, 2 when connected
	return ( PORT_IS_ON(STATUS_PORT,STATUS_PIN)/(STATUS_PIN*2) );
}

int SIM_Presence(void) {  //4 when not connected, 0 when connected
	return ( PORT_IS_ON(SIM_PRESENCE_PORT,SIM_PRESENCE_PIN) / (SIM_PRESENCE_PIN*2) );
}

int SIM_Status(void) { // returns 0 if good, 4 if not good
	int response[16];
	rprintfInit(SendByte); rprintf("AT+CIMI\r"); FlushBuffer();

	for (int i=0;i<3;++i) {  // get three byte reply
		response[i]=GetByte(1);
		if (response[i] == TIMEOUT_ERROR) 
			return ERROR;
	} //end for
	if ( response[2] == 43) { // If response[2] is +, then its a +CME Error
		for (int i=0;i<15;++i) {  
			response[i]=GetByte(1);
			if (response[i] == TIMEOUT_ERROR) 
				return ERROR;
		} //end for
		return ERROR;
	} //end if
	else {	// Otherwise, your SIM card is all set and ready
		for (int i=0;i<17;++i) {  // get the SIM IMSI number
			response[i]=GetByte(1);
			if (response[i] == TIMEOUT_ERROR)
				return ERROR;
		} //end for
		// TODO: USE THE DIGITS HERE SOMEHOW???
		return OK; // returns '0' , OK
	} //end else
} //end sub 


int DroneCell_InternetSettings(void) {  
	if (CIPHEAD() != OK)  // Enable IPD header on receive packets
		return CIPHEAD_ERROR;
	delay_ms(10); // pause 10mS to be safe and let it initialize
		if (CSTT() != OK )  // Define IP context and WAP name
		return CSTT_ERROR;

/*	if (CGDCONT() != OK )  // Configure WAP settings - Not really needed for Internets
		return CSTT_ERROR;
*/
	return OK;
}//end sub


int Start_Transmission(void) {
	int response[2];
	rprintfInit(SendByte); rprintf("AT+CIPSEND\r");  FlushBuffer();

	for (int i=0;i<3;++i) {  // 13 10  62  32 
		response[i]=GetByte(4);
		if (response[i] == TIMEOUT_ERROR) 
			return TIMEOUT_ERROR;
	} //end for

	if ( response[2] != 62 ) 
		return ERROR;
	else 
		return OK;
} //end sub

int End_Transmission(void) {
	int response[7];
	SendByte(26);  // Ctrl+z or END TRANSMIT code

	for (int i=0;i<4;++i) {  // Get the SEND OK response , this was 7
		response[i]=GetByte(10); 
		if (response[i] == TIMEOUT_ERROR)
			return TIMEOUT_ERROR;
	} //end for

	if (response[3] == 83) {  
		for (int i=0;i<8;++i) {  // get the rest of the bytes here
			response[i]=GetByte(1);
			if (response[i] == TIMEOUT_ERROR)
				return TIMEOUT_ERROR;
		} //end for
		return OK;
	} //end if
	else 
		return ERROR;
} //end sub


int Close_Socket(void) {
	int response[3];
 	rprintfInit(SendByte); rprintf("AT+CIPCLOSE\r"); FlushBuffer();
	
	for (int i=0;i<4;++i) {  // Get the SEND OK response , this was 7
		response[i]=GetByte(10); 
		if (response[i] == TIMEOUT_ERROR)
			return TIMEOUT_ERROR;
	} //end for
	
	if (response[2] == 55)
		return OK;
	else 
		return ERROR;
} // end sub

int DroneCell_PowerOn(void) {
	LED_off();
	Toggle_PWRKEY(); // Toggle the PWRKEY, turning on the DroneCell
	int counter=0;
	while (Read_STATUS() == LOW && counter < 2) { // if the STATUS pin is LOW
		rprintfInit(DebugPrint); rprintf("Error: DroneCell not on\n");
		Toggle_PWRKEY(); // Toggle the PWRKEY, turning off the DroneCell
		counter++;
		delay_ms(1500); // delay 1.5 second
	} // end while
		
	if (counter >= 2) { // if Powering up didnt work
		rprintfInit(DebugPrint); rprintf("Error: DroneCell not poweron\n");
		return POWER_ERROR;
	}
		
	delay_ms(5000); // delay 5 seconds for everything to initialize
	
	rprintfInit(SendByte);rprintf("AT\r");FlushBuffer();
	int response[1];
	for (int i=0;i<2;i++) {
		response[i] = GetByte(1);
		if ( response[i] == TIMEOUT_ERROR )
			return POWER_ERROR;
	}

		if (SIM_Status() != OK)   // if SIM Card is not detected
			return SIM_ERROR;
		else 	//Power up and SIM card was perfect
			return OK;
} //end int DroneCell

int DroneCell_PowerOff(void) {
	int response[37];
	rprintfInit(SendByte); rprintf("AT+CPOWD=1\r"); FlushBuffer();
	for (int i=0;i<38;++i) {  // Get the headings and everything
		response[i]=GetByte(1);
		if (response[i] == TIMEOUT_ERROR) 
				return ERROR;
	} //end for
	return OK;
}

int DroneCell_ConnectCalling(void) {
// Waits until you have a signal, the Status light will flash less frequently when connected
	unsigned char connect_counter=0;

	while( Connect_Status() != CONNECT && connect_counter<50) { 
		delay_ms(1000);
		connect_counter++;
	}
	if (connect_counter >= 50 ) // cant connect for more than 50 seconds
		return CONNECT_ERROR;

	return OK;
} //end sub

int DroneCell_ConnectInternet(void) {
	int ErrorCode;
	 // Set CSTT and CGDCONT network settings - necessary for internet
	ErrorCode = DroneCell_InternetSettings();
	if ( ErrorCode != OK) 
		return ErrorCode;
	
	// Waits until you have a signal, the Status light will flash less frequently when connected
	int	connect_counter=0;
	while( (Connect_Status() != CONNECT) && (connect_counter<50) ) { 
		delay_ms(1000);
		connect_counter++;
	}
	if (connect_counter >= 50 ) // cant connect for more than 50 seconds
		return CONNECT_ERROR;
	delay_ms(75); // pause 10mS to be safe and let it initialize
	// Attach to Cellular Network for Internet Usage 
	if ( CIICR() != OK )
		return ATTACH_ERROR;
	
	delay_ms(75); // pause 10mS to be safe and let it initialize
	if ( CIFSR() != OK ) 
		return IP_ERROR;

	delay_ms(75); // pause 10mS to be safe and let it initialize
	return OK; // everything was fine

} //end sub

// --- NEW TERRITORY ----
int DroneCell_Call(char phone_number[]) { // DTMF tone dial out
	rprintfInit(SendByte);
	rprintf("ATD"); rprintfStr(phone_number); rprintf(";\r");
	rprintfInit(uart1SendByte);
	int data,i=0;
	// TODO: HANDLE ERROR CHECKING AND BYTES RECEIVED
	while(1) { 
		data= GetByte(10);
		i++;
		rprintf("%d =  %c or %d \n", i , data, data);
	}
}


int CMGF(void) {
	int response[1];
	rprintfInit(SendByte); rprintf("AT+CMGF=1\r"); FlushBuffer();
	for (int i=0;i<2;++i) { 
		response[i]=GetByte(2); 
		if (response[i] == TIMEOUT_ERROR)
			return TIMEOUT_ERROR;
	} //end for
	return response[0];
}

int CSCS(void) {
	int response[1];
	rprintfInit(SendByte); rprintf("AT+CSCS=\"GSM\"\r");
	//rprintfStr(type);rprintf("\"\r"); 
	FlushBuffer();
	for (int i=0;i<2;++i) { 
		response[i]=GetByte(2); 
		if (response[i] == TIMEOUT_ERROR)
			return TIMEOUT_ERROR;
	} //end for
	return response[0];
}

int CSCA(char msgcenter[]) {
	int response[18];

	rprintfInit(SendByte); 
	rprintf("AT+CSCA=\"");rprintfStr(msgcenter);rprintf("\"\r");

	for (int i=0;i<2;++i) { 
		response[i]=GetByte(2);
		if (response[i] == TIMEOUT_ERROR)
			return TIMEOUT_ERROR;
	} //end for
	return response[0];
}

int CSMP(void) {
	int response[1];
	rprintfInit(SendByte); rprintf("AT+CSMP=17,167,0,240\r"); FlushBuffer();

	for (int i=0;i<2;++i) { 
		response[i]=GetByte(1); 
		if (response[i] == TIMEOUT_ERROR)
			return TIMEOUT_ERROR;
	} //end for
	return response[0];
}

int DroneCell_TextInit(void) { 
	int ErrorCode;
	// delays seem to be necessary, havent optimized this yet
	ErrorCode = CMGF();
	if ( ErrorCode != OK ) 
		return CMGF_ERROR;
	delay_ms(1000);
 
	ErrorCode = CSCS();
	if ( ErrorCode != OK ) 
		return CSCS_ERROR;
	delay_ms(1000); 

	ErrorCode = CSCA(TXT_MSG_CENTER);
	if ( ErrorCode != OK ) 
		return ErrorCode;
	delay_ms(1000);

	ErrorCode = CSMP();
	if ( ErrorCode != OK ) 
		return CSMP_ERROR; 
				
	return OK;
}

int Text_Transmit(char phone_number[]) {
	int response[2];
	rprintfInit(SendByte); rprintf("AT+CMGS=\"");
	rprintfStr(phone_number); rprintf("\"\r");  FlushBuffer();

	for (int i=0;i<3;++i) {  // 13 10  62  32 
		response[i]=GetByte(4);
		if (response[i] == TIMEOUT_ERROR) 
			return TIMEOUT_ERROR;
	} //end for

	if ( response[2] != 62 ) 
		return ERROR;
	else 
		return OK;
} //end sub

int DroneCell_Text( char phone_number[], char message[] ) {
	int ErrorCode; int response[11];
	ErrorCode= Text_Transmit(phone_number);  // Start text transmission
	if ( ErrorCode != OK) 
		return ERROR;
	rprintfInit(SendByte);
	rprintfStr(message);
	SendByte(26); // send Ctrl+Z to terminate transmission
	// NEEDS ERROR CHECKING BELOW HERE
	for (int i=0;i<10;++i) { 
		response[i]=GetByte(4);
		if (response[i] == TIMEOUT_ERROR) 
			return TIMEOUT_ERROR;
	} //end for
	
	while ( GetByte(2) != 13 );

	return OK;
}
